==> xdbgen-1.0.0.tar.gz: lib/src/xdbgen_base.dart <==
// TODO: Put public facing types in this file.
import 'dart:convert';
import 'dart:io';
import 'dart:mirrors';

import 'package:path/path.dart';
import 'package:xdatabase/xdatabase.dart';

class Migration {
  List<Type> tables;
  DeclarationHolder holder = DeclarationHolder();
  Migration({required this.tables}) {
    for (var tb in tables) {
      registTable(tb);
    }
  }

  void registTable(Type tp) {
    final cm = reflectClass(tp);

    var tableName = '';
    final className = tp.toString();
    var indexes = <IndexDeclaration>[];
    var uniqueIndexes = <UniqueDeclaration>[];
    var fields = <FieldDeclaration>[];

    cm.metadata.forEach((InstanceMirror element) {
      if (element.reflectee is Table) {
        final table = element.reflectee as Table;
        tableName = table.tableName;
      }
      if (element.reflectee is Index) {
        final index = element.reflectee as Index;
        indexes.add(
            IndexDeclaration(indexName: index.name, fields: index.columns));
      }
      if (element.reflectee is UniqueIndex) {
        final unique = element.reflectee as UniqueIndex;
        uniqueIndexes.add(
            UniqueDeclaration(indexName: unique.name, fields: unique.columns));
      }
    });

    cm.declarations.forEach((Symbol key, DeclarationMirror value) {
      value.metadata.forEach((InstanceMirror element) {
        if (element.reflectee is Field) {
          final field = element.reflectee as Field;
          fields.add(FieldDeclaration(
              symbol: key,
              propertyName: symbolToName(key),
              fieldName: nameUnderLine(symbolToName(key)),
              fieldType: field.type,
              primary: field.primary,
              mark: field.mark));
        }
      });
    });
    var table = TableDeclaration(tableName: tableName, className: className);
    table.fields.addAll(fields);
    table.indexes.addAll(indexes);
    table.uniqueIndexes.addAll(uniqueIndexes);
    holder.declarations.tables.addEntries([MapEntry(tableName, table)]);
    holder.tables.addEntries([MapEntry(tp, table)]);
  }

  String get workingDirectory {
    return dirname(Platform.script.toString()).substring(7);
  }

  String get versionsDirectory {
    final basePath = workingDirectory;
    final dirPath = join(basePath, 'versions');
    final dir = Directory(dirPath);
    if (!dir.existsSync()) {
      dir.createSync();
    }
    return dirPath;
  }

  String versionInFilePath(String filePath) {
    final fileName = filePath.split(separator).last;
    return fileName.split('.').first;
  }

  String get latestVersionPath {
    final dir = Directory(versionsDirectory);
    final fileList = dir.listSync();
    final versions = fileList
        .where((f) => f.path.endsWith('.map'))
        .map((f) => versionInFilePath(f.path))
        .toList();
    versions.sort((v1, v2) => int.parse(v1).compareTo(int.parse(v2)));
    final matched = fileList
        .where((element) => element.path.endsWith('.map'))
        .where((element) => versionInFilePath(element.path) == versions.last)
        .map((e) => e.path)
        .toList();
    return matched.isNotEmpty ? matched.first : '';
  }

  String get currentUpgradeSqlPath {
    return join(versionsDirectory, 'current.sql');
  }

  String get currentVersionPath {
    var fileName = newVersion();
    return join(versionsDirectory, '$fileName.map');
  }

  String targetVersionUpgradeSQLPath(String versionPath) {
    final ver = versionInFilePath(versionPath);
    return join(versionsDirectory, '$ver.sql');
  }

  int newVersion() {
    final now = DateTime.now();
    final fileName = now.year * 10000000000 +
        now.month * 100000000 +
        now.day * 1000000 +
        now.hour * 10000 +
        now.minute * 100 +
        now.second;
    return fileName;
  }

  void diffVersion() {
    print('newest path:$latestVersionPath');
    final sql = DeclarationHolder().compareWithPath(latestVersionPath);
    final f = File(currentUpgradeSqlPath);
    f.writeAsStringSync(sql);
  }

  void releaseVersion() {
    final vpath = currentVersionPath;
    DeclarationHolder().saveToPath(vpath);
    File(currentUpgradeSqlPath).copySync(targetVersionUpgradeSQLPath(vpath));
    File(currentUpgradeSqlPath).deleteSync();
  }

  void generateFiles() {
    final tableDeclaration = jsonEncode(holder.declarations.asMap());
    final lines = <String>[];
    lines.add('import \'package:xdatabase/xdatabase.dart\';\n');
    lines.add('import \'models.dart\';\n\n');
    holder.declarations.tables.forEach((tableName, tableDeclaration) {
      final insName = tableDeclaration.className.toLowerCase();
      final className = tableDeclaration.className;
      lines.add('class ${className}Beean implements Bean<$className> {');
      lines.add('  $className? $insName;');
      lines.add('  ${className}Bean(this.$insName);');
      lines.add('  @override');
      lines.add('  Map<String, dynamic> asMap() {');
      lines.add('    if ($insName == null) {');
      lines.add('      throw UnsupportedError(\'Not Initialized\');');
      lines.add('    }');
      lines.add('    return {');
      tableDeclaration.fields.forEach((field) {
        lines.add(
            '      \'${field.fieldName}\': $insName!.${field.propertyName},');
      });
      lines.add('    };');
      lines.add('  }\n');
      lines.add('  @override');
      lines.add('  $className fromMap(Map<String, dynamic> map) {');
      lines.add('    final $insName = $className();');
      tableDeclaration.fields.forEach((field) {
        lines.add(
            '    $insName.${field.propertyName} = map[\'${field.fieldName}\'];');
      });
      lines.add('    return $insName;');
      lines.add('  }\n');
      lines.add('  @override');
      lines.add('  Future<int> delete(String statement, QueryAble conn) {');
      lines.add(
          '    return Orm<$className>(conn).query(statement).delete(this);');
      lines.add('  }\n');
      lines.add('  @override');
      lines.add('  Future<int> deleteByPk(QueryAble conn) {');
      lines.add('    return Orm<$className>(conn).deleteByPk(this);');
      lines.add('  }\n');
      lines.add('  @override');
      lines.add('  Future<int> insert(QueryAble conn) {');
      lines.add('    return Orm<$className>(conn).insert(this);');
      lines.add('  }\n');
      lines.add('  @override');
      lines.add(
          '  Future<List<$className>> select(String statement, QueryAble conn) {');
      lines.add(
          '    return Orm<$className>(conn).query(statement).fetch(this, all: log != null);');
      lines.add('  }\n');
      lines.add('  @override');
      lines.add('  Future<$className?> selectByPk(QueryAble conn) {');
      lines.add('    return Orm<$className>(conn).byPrimary(this);');
      lines.add('  }\n');
      lines.add('  @override');
      lines.add('  Future<int> update(String statement, QueryAble conn) {');
      lines.add(
          '    return Orm<$className>(conn).query(statement).update(this);');
      lines.add('  }\n');
      lines.add('  @override');
      lines.add('  Future<int> updateByPK(QueryAble conn) {');
      lines.add('    return Orm<$className>(conn).updateByPk(this);');
      lines.add('  }');
      lines.add('}\n');
    });
    lines.add('');
    lines.add('');
    lines.add('const String declarationMap = \'\'\'$tableDeclaration\'\'\'');
    final seqs = workingDirectory.split(separator);
    final basePath = seqs.getRange(0, seqs.length - 1).join(separator);
    print('basePath:$basePath');
    final filePath = '${basePath}/lib/src/modelBeans.dart';
    print('filePath:$filePath');
    File(filePath).writeAsStringSync(lines.join('\n'));
  }

  void executeCommand(String command) {
    switch (command.toLowerCase()) {
      case 'diff':
        diffVersion();
        break;
      case 'release':
        releaseVersion();
        break;
      case 'gen':
        generateFiles();
        break;
      default:
        print('command in (diff, release)');
        break;
    }
  }
}

==> xdbgen-1.0.1.tar.gz: lib/src/xdbgen_base.dart <==
// TODO: Put public facing types in this file.
import 'dart:convert';
import 'dart:io';
import 'dart:mirrors';

import 'package:path/path.dart';
import 'package:xdatabase/xdatabase.dart';

class Migration {
  List<Type> tables;
  DeclarationHolder holder = DeclarationHolder();
  Migration({required this.tables}) {
    for (var tb in tables) {
      registTable(tb);
    }
  }

  void registTable(Type tp) {
    final cm = reflectClass(tp);

    var tableName = '';
    final className = tp.toString();
    var indexes = <IndexDeclaration>[];
    var uniqueIndexes = <UniqueDeclaration>[];
    var fields = <FieldDeclaration>[];

    cm.metadata.forEach((InstanceMirror element) {
      if (element.reflectee is Table) {
        final table = element.reflectee as Table;
        tableName = table.tableName;
      }
      if (element.reflectee is Index) {
        final index = element.reflectee as Index;
        indexes.add(
            IndexDeclaration(indexName: index.name, fields: index.columns));
      }
      if (element.reflectee is UniqueIndex) {
        final unique = element.reflectee as UniqueIndex;
        uniqueIndexes.add(
            UniqueDeclaration(indexName: unique.name, fields: unique.columns));
      }
    });

    cm.declarations.forEach((Symbol key, DeclarationMirror value) {
      value.metadata.forEach((InstanceMirror element) {
        if (element.reflectee is Field) {
          final field = element.reflectee as Field;
          fields.add(FieldDeclaration(
              symbol: key,
              propertyName: symbolToName(key),
              fieldName: nameUnderLine(symbolToName(key)),
              fieldType: field.type,
              primary: field.primary,
              mark: field.mark));
        }
      });
    });
    var table = TableDeclaration(tableName: tableName, className: className);
    table.fields.addAll(fields);
    table.indexes.addAll(indexes);
    table.uniqueIndexes.addAll(uniqueIndexes);
    holder.declarations.tables.addEntries([MapEntry(tableName, table)]);
    holder.tables.addEntries([MapEntry(tp, table)]);
  }

  String get workingDirectory {
    return dirname(Platform.script.toString()).substring(7);
  }

  String get versionsDirectory {
    final basePath = workingDirectory;
    final dirPath = join(basePath, 'versions');
    final dir = Directory(dirPath);
    if (!dir.existsSync()) {
      dir.createSync();
    }
    return dirPath;
  }

  String versionInFilePath(String filePath) {
    final fileName = filePath.split(separator).last;
    return fileName.split('.').first;
  }

  String get latestVersionPath {
    final dir = Directory(versionsDirectory);
    final fileList = dir.listSync();
    final versions = fileList
        .where((f) => f.path.endsWith('.map'))
        .map((f) => versionInFilePath(f.path))
        .toList();
    versions.sort((v1, v2) => int.parse(v1).compareTo(int.parse(v2)));
    final matched = fileList
        .where((element) => element.path.endsWith('.map'))
        .where((element) => versionInFilePath(element.path) == versions.last)
        .map((e) => e.path)
        .toList();
    return matched.isNotEmpty ? matched.first : '';
  }

  String get currentUpgradeSqlPath {
    return join(versionsDirectory, 'current.sql');
  }

  String get currentVersionPath {
    var fileName = newVersion();
    return join(versionsDirectory, '$fileName.map');
  }

  String targetVersionUpgradeSQLPath(String versionPath) {
    final ver = versionInFilePath(versionPath);
    return join(versionsDirectory, '$ver.sql');
  }

  int newVersion() {
    final now = DateTime.now();
    final fileName = now.year * 10000000000 +
        now.month * 100000000 +
        now.day * 1000000 +
        now.hour * 10000 +
        now.minute * 100 +
        now.second;
    return fileName;
  }

  void diffVersion() {
    print('newest path:$latestVersionPath');
    final sql = DeclarationHolder().compareWithPath(latestVersionPath);
    final f = File(currentUpgradeSqlPath);
    f.writeAsStringSync(sql);
  }

  void releaseVersion() {
    final vpath = currentVersionPath;
    DeclarationHolder().saveToPath(vpath);
    File(currentUpgradeSqlPath).copySync(targetVersionUpgradeSQLPath(vpath));
    File(currentUpgradeSqlPath).deleteSync();
  }

  void generateFiles() {
    final tableDeclaration = jsonEncode(holder.declarations.asMap());
    final lines = <String>[];
    lines.add('import \'package:xdatabase/xdatabase.dart\';\n');
    lines.add('import \'models.dart\';\n\n');
    holder.declarations.tables.forEach((tableName, tableDeclaration) {
      final insName = tableDeclaration.className.toLowerCase();
      final className = tableDeclaration.className;
      lines.add('class ${className}Beean implements Bean<$className> {');
      lines.add('  $className? $insName;');
      lines.add('  ${className}Bean(this.$insName);');
      lines.add('  @override');
      lines.add('  Map<String, dynamic> asMap() {');
      lines.add('    if ($insName == null) {');
      lines.add('      throw UnsupportedError(\'Not Initialized\');');
      lines.add('    }');
      lines.add('    return {');
      tableDeclaration.fields.forEach((field) {
        lines.add(
            '      \'${field.fieldName}\': $insName!.${field.propertyName},');
      });
      lines.add('    };');
      lines.add('  }\n');
      lines.add('  @override');
      lines.add('  $className fromMap(Map<String, dynamic> map) {');
      lines.add('    final $insName = $className();');
      tableDeclaration.fields.forEach((field) {
        lines.add(
            '    $insName.${field.propertyName} = map[\'${field.fieldName}\'];');
      });
      lines.add('    return $insName;');
      lines.add('  }\n');
      lines.add('  @override');
      lines.add('  Future<int> delete(String statement, QueryAble conn) {');
      lines.add(
          '    return Orm<$className>(conn).query(statement).delete(this);');
      lines.add('  }\n');
      lines.add('  @override');
      lines.add('  Future<int> deleteByPk(QueryAble conn) {');
      lines.add('    return Orm<$className>(conn).deleteByPk(this);');
      lines.add('  }\n');
      lines.add('  @override');
      lines.add('  Future<int> insert(QueryAble conn) {');
      lines.add('    return Orm<$className>(conn).insert(this);');
      lines.add('  }\n');
      lines.add('  @override');
      lines.add(
          '  Future<List<$className>> select(String statement, QueryAble conn) {');
      lines.add(
          '    return Orm<$className>(conn).query(statement).fetch(this, all: log != null);');
      lines.add('  }\n');
      lines.add('  @override');
      lines.add('  Future<$className?> selectByPk(QueryAble conn) {');
      lines.add('    return Orm<$className>(conn).byPrimary(this);');
      lines.add('  }\n');
      lines.add('  @override');
      lines.add('  Future<int> update(String statement, QueryAble conn) {');
      lines.add(
          '    return Orm<$className>(conn).query(statement).update(this);');
      lines.add('  }\n');
      lines.add('  @override');
      lines.add('  Future<int> updateByPK(QueryAble conn) {');
      lines.add('    return Orm<$className>(conn).updateByPk(this);');
      lines.add('  }');
      lines.add('}\n');
    });
    lines.add('');
    lines.add('');
    lines.add('const String declarationMap = \'\'\'$tableDeclaration\'\'\'');
    final seqs = workingDirectory.split(separator);
    final basePath = seqs.getRange(0, seqs.length - 1).join(separator);
    print('basePath:$basePath');
    final filePath = '${basePath}/lib/modelBeans.dart';
    print('filePath:$filePath');
    File(filePath).writeAsStringSync(lines.join('\n'));
  }

  void executeCommand(String command) {
    switch (command.toLowerCase()) {
      case 'diff':
        diffVersion();
        break;
      case 'release':
        releaseVersion();
        break;
      case 'gen':
        generateFiles();
        break;
      default:
        print('command in (diff, release)');
        break;
    }
  }
}
